clear all; close all; clc;

% Modle H(x,s), equation (E2.2)
w0  = 3;
m   = .5;
H   = @(s,x) exp(-x^2.*s).*sqrt(pi)./(sqrt(s))./(s.^2/w0^2+m.*s/w0+1);
% Axe de l'onde (L=3)
L   = 3;
dx  = .05;
xdx = 0:dx:L;
% Calcul de la rponse frquentielle de H(L,s)
W            = logspace(-2,1,500)*2*pi;
nW           = numel(W);
Hf(1,1,1:nW) = H(complex(0,W),L); 
% Approximation rationelle exacte de H(L,s) par mthode de Loewner
% utilisant la Toolbox MOR
red          = [];
opt.verbose  = true;
HrL          = mor.lti({W,Hf},red,opt);
% Comparaison des rponses frquentielles
figure
subplot(211); mor.bode({Hf},'b-',HrL,'r--',W)
l=legend('$H(L,s)$','$\hat{H}(L,s)$');
set(l,'interpreter','latex','fontsize',26)
subplot(212); mor.bode({Hf},'b-',HrL,'r--',W,'phase')
% Calcul des la rponses frquentielles de H(x,s) pour plusieurs valeurs de
% x, considres comme des sorties 
for ii = 1:numel(xdx)
    xi = xdx(ii);
    Hf(ii,1,1:numel(W)) = H(complex(0,W),xi);
end
% Approximation rationelle r=50 de H(x,s) par mthode de Loewner utilisant
% la Toolbox MOR
red            = 50;
opt.ensureStab = true;
Hr             = mor.lti({W,Hf},red,opt);
% Simulation temporelle de Hr(x,s) en rponse  une impulsion
Tf  = 10;
dt  = 2e-2;
tdt = 0:dt:Tf;
% Simulation du modle approch Hr(x,s)
yr  = impulse(Hr,tdt);
% Simulation de la solution exacte de (E2.1)
u   = zeros(1,length(xdx));
y   = zeros(numel(tdt),numel(xdx));
for tt = 1:numel(tdt)
    for ii = 1:numel(xdx)
        time = tdt(tt)-xdx(ii)^2; y(tt,ii) = 1/sqrt(time);
         if isnan(y(tt,ii)) || isinf(y(tt,ii)) || time < 0
            y(tt,ii)  = 0;
            yr(tt,ii) = 0;
        end
    end
end
for ii = 1:numel(xdx)
    y(:,ii) = lsim(tf(1,[1/w0^2 m/w0 1]),y(:,ii),tdt);
end
% Comparaison des rponses impulsionelles 
% * bleu  : solution exacte (mais sujette au bruit numrique)
% * rouge : solution du modle rationnel approximt Hr(x,s) (plus stable numriquement)
figure
for tt = 1:numel(tdt)
    plot(xdx,y(tt,:),'b-',xdx,yr(tt,:),'ro-')
    title(['\bf{t = ' num2str(tdt(tt)) '}'],'interpreter','latex','fontsize',26)
    set(gca,'Ylim',[min(yr(:)) max(yr(:))])
    xlabel('$\bf{x [m]}$','interpreter','latex','fontsize',26)
    ylabel('$\bf{y(x,t)}$','interpreter','latex','fontsize',26)
    drawnow
end
% On note des singularits forte de la solution exacte, moins efficace que
% l'intgration numrique du modle rduit
