clear all
close all
clc

%% Chargement du modèle initial
load modelClampedBeam
H = sysFEM(end,1); % On ne considère que la dernière sortie
n = length(H.A); % Dimension du modèle initial

%% Changement de base pour la base modale
[P, Ab] = eig(H.A);
Bb      = P \ H.B;
Cb      = H.C * P;
Db      = H.D;

% Réalisation modale complexe
Hb      =  ss(Ab, Bb, Cb, Db);

%% Calcul des ratios et tri des valeurs propres
lambda = diag(Ab);
for i=1:length(H.A)
    ratio(i) = norm(Cb(:,i) * Bb(i,:),2) / abs(real(lambda(i)));
end
[ratio, ind] = sort(ratio, 'descend');

%% Troncature de la réalisation modale
r   = 10;
xi1 = ind(1:r);

% Réalisation modale complexe du modèle réduit
Hrc =  ss(Ab(xi1,xi1),Bb(xi1,:),Cb(:,xi1),Db);

%% Changement de base du modèle réduit pour une réalisation réelle
Pr      = []; % Matrice de changement de base
lambdar = diag(Hrc.A);
i       = 0;
while i < r
    i = i+1;
    if imag(lambdar(i)) == 0
        Pr  = blkdiag(Pr, 1);
    else
        Pr  = blkdiag(Pr,[1 1i;1,-1i]);
        i   = i+1;
    end
end
% Application du changement de base
Hrr  =  ss(real(Pr\Hrc.A*Pr), real(Pr\Hrc.B), real(Hrc.C * Pr), Hrc.D);

%% Calcule des bornes et normes de l'erreur
% Réorganisation de l'ordre des états
Ab      = Ab(ind,ind);
Bb      = Bb(ind,:);
Cb      = Cb(:,ind);
lambda  = diag(Ab);
% Borne Hinf
hi_bound = 0;
for i = r+1:n
    hi_bound = hi_bound  + norm(Cb(:,i) * Bb(i,:),2) / abs(real(lambda(i)));
end
hi_error = norm(H - Hrr, inf);
[hi_error, hi_bound]/norm(H,inf) * 100
% Erreur H2
h2_error_res    = 0;
for i = r+1:n 
    phi_i       = Cb(:,i) * Bb(i,:);
    lambda_i    = lambda(i);
    for k = r+1:n
        phi_k           = Cb(:,k) * Bb(k,:);
        lambda_k        = lambda(k);
        h2_error_res    = h2_error_res  -  trace(phi_i * phi_k.')/(lambda_i + lambda_k);
    end
end
h2_error_res    = sqrt(real(h2_error_res));
h2_error        = norm(H - Hrr, 2);
[h2_error_res, h2_error]/norm(H,2) * 100

%% Comparaison des pôles
figure
pos = imag(lambda) >=0;
loglog(-real(lambda(pos)),imag(lambda(pos)),'b.')
hold on
pos = imag(lambdar) >=0;
loglog(-real(lambdar(pos)),imag(lambdar(pos)),'r.');
set(gca(),'fontsize',20);
xlabel('$-Re(\lambda)$','interpreter','latex','fontsize',26);
ylabel('$Im(\lambda)$','interpreter','latex','fontsize',26);

l = legend('P\^oles de $H_{900}$','P\^oles de $\hat{H}$','Location','SouthEast');
set(l,'interpreter','latex','fontsize',26)
grid on

%% Comparaison des réponses fréquentielles
figure
wmin = 1;
wmax = 1e4;
w = logspace(log10(wmin),log10(wmax),300);
Mh    = bode(H,w);
Mhr         = bode(Hrr, w);

semilogx(w/(2*pi), 20*log(abs(squeeze(Mh))),'b');
hold on
semilogx(w/(2*pi), 20*log(abs(squeeze(Mhr))),'r--');
set(gca(),'fontsize',20);
set(gca(),'xLim',[wmin, wmax]/(2*pi));
l = legend('$H_{900}$','$\hat{H}$');
set(l,'interpreter','latex','fontsize',26);
xlabel('Fr\''equence (Hz)','interpreter','latex','fontsize',26);
ylabel('Gain (dB)','interpreter','latex','fontsize',26);
grid on


