clear all
close all
clc

%% Chargement du modèle initial
load modelClampedBeam
H = sysFEM(end,1); % On ne considère que la dernière sortie
n = length(H.A); % Dimension du modèle initial

%% Changement de base pour la base équilibrée
[Hb, sigma] = balreal(H);

% Dans cette base, les gramiens sont égaux et diagonaux
P = gram(Hb, 'c');
Q = gram(Hb, 'o');
norm(P - Q)
norm(diag(P) - sigma)

%%
% La décroissance des valeurs singulières donne une estimation de l'ordre
% de réduction r adéquat
hi_bound_cumulated = [];

for i = 1:n
    hi_bound_cumulated(i) = 2 * sum(sigma(i+1:end));
end
figure
ax = plotyy(1:n,sigma/max(sigma),1:n,hi_bound_cumulated/norm(H,inf) *100,@semilogy,@semilogy); grid on;
set(gca(),'fontsize',20);
set(ax(1),'Yscale','log','fontsize',20)
set(ax(2),'Yscale','log','fontsize',20)
ylabel(ax(1),'$\frac{\sigma_i}{\sigma_{max}}$','interpreter','latex','fontsize',26)
ylabel(ax(2),'$\frac{\overline{e}_{i,\infty}}{\Vert H_{900}\Vert_{\mathcal{H}_\infty}} (\%)$','interpreter','latex','fontsize',26)
xlabel('$i$','interpreter','latex','fontsize',26)
grid on
yl = get(ax(1),'Ylim');
hold on
plot([10,10],yl,'k--');

%% Troncature de la réalisation équilibrée
r   = 10;
xi1 = 1:r;
Hr  = ss(Hb.A(xi1,xi1), Hb.B(xi1,:), Hb.C(:,xi1), Hb.D);

%% Calcul de la norme hinf 
hi_bound = 2 * sum(sigma(r+1:end));
hi_error = norm(H-Hr,inf);

[hi_error, hi_bound] / norm(H, inf) * 100

h2_error = norm(H - Hr, 2);
h2_error / norm(H, 2) * 100

%% Réponses fréquentielles des modèles
figure
wmin = 1;
wmax = 1e4;
w = logspace(log10(wmin),log10(wmax),300);
Mh    = bode(H,w);
Mhr         = bode(Hr, w);

semilogx(w/(2*pi), 20*log(abs(squeeze(Mh))),'b');
hold on
semilogx(w/(2*pi), 20*log(abs(squeeze(Mhr))),'r--');
set(gca(),'fontsize',20);
set(gca(),'xLim',[wmin, wmax]/(2*pi));
l = legend('$H_{900}$','$\hat{H}$');
set(l,'interpreter','latex','fontsize',26);
xlabel('Fr\''equence (Hz)','interpreter','latex','fontsize',26);
ylabel('Gain (dB)','interpreter','latex','fontsize',26);
grid on


