clear all, close all, clc

%% Chargement du modele initial
load modelClampedBeam
H = sysFEM(end,1); % On ne considre que la dernire sortie

%% Dfinition du maillage frquentiel et calcul de la rponse frquentielle
W  = logspace(0,4,300);
FR = freqresp(H,W);

%% Interpolation exacte 
r         = []; % On laisse l'algorithme dterminer l'ordre minimal
[Hr,info] = mor.lti({W,FR},[]);

%% Rsultats
figure
wmin = 1;
wmax = 1e4;
w    = logspace(log10(wmin),log10(wmax),300);
Mh   = bode(H,w);
Mhr  = bode(Hr, w);
semilogx(w/(2*pi), 20*log(abs(squeeze(Mh))),'b.');
hold on
semilogx(w/(2*pi), 20*log(abs(squeeze(Mhr))),'r-');
set(gca(),'fontsize',20);
set(gca(),'xLim',[wmin, wmax]/(2*pi));
ll = legend('$H_{900}(\imath \omega_i)$','$\hat{H}$');
set(ll,'interpreter','latex','fontsize',26,'Location','SouthWest');
xlabel('Fr\''equence (Hz)','interpreter','latex','fontsize',26);
ylabel('Gain (dB)','interpreter','latex','fontsize',26);
grid on

%% Approximation
r         = 10; % On force l'approximation avec un ordre 10
[Hr,info] = mor.lti({W,FR},r);
100*norm(H-Hr,inf)/norm(H,inf)
100*norm(H-Hr,2)/norm(H,2)

%% Rsultats
figure
wmin = 1;
wmax = 1e4;
w    = logspace(log10(wmin),log10(wmax),300);
Mh   = bode(H,w);
Mhr  = bode(Hr, w);
semilogx(w/(2*pi), 20*log(abs(squeeze(Mh))),'b.');
hold on
semilogx(w/(2*pi), 20*log(abs(squeeze(Mhr))),'r-');
set(gca(),'fontsize',20);
set(gca(),'xLim',[wmin, wmax]/(2*pi));
ll = legend('$H_{900}(\imath \omega_i)$','$\hat{H}$');
set(ll,'interpreter','latex','fontsize',26,'Location','SouthWest');
xlabel('Fr\''equence (Hz)','interpreter','latex','fontsize',26);
ylabel('Gain (dB)','interpreter','latex','fontsize',26);
grid on